@@ -2,13 +2,16 @@ import logging
 import pprint
 import time
 from collections import namedtuple
+from datetime import timedelta
 from distutils.version import StrictVersion
 from queue import Empty
 from random import randint
 
+import requests
 import soco
 import soco.alarms
 import tabulate
+import xmltodict
 
 from .speaker_info import print_speaker_table
 from .utils import (
@@ -271,23 +274,38 @@ def print_info(speaker, action, args, soco_function, use_local_speaker_list):
 
 @zero_parameters
 def track(speaker, action, args, soco_function, use_local_speaker_list):
+    state = speaker.get_current_transport_info()["current_transport_state"].lower()
     if speaker.is_playing_line_in:
-        print("Playing from Line In")
+        print("Using Line In (state: {})".format(state))
     else:
-        output = speaker.get_current_track_info()
-        if output["duration"] == "0:00:00":
-            if (
-                speaker.get_current_transport_info()["current_transport_state"]
-                != "PLAYING"
-            ):
-                print("Stream Stopped: No Track Playing")
+        track_info = speaker.get_current_track_info()
+        logging.info("Current track info:\n{}".format(track_info))
+        # Stream
+        if track_info["duration"] == "0:00:00":
+            if track_info["artist"] != "":
+                print("Stream is {}".format(state))
+                for item in sorted(track_info):
+                    if item not in [
+                        "metadata",
+                        "album_art",
+                        "duration",
+                        "playlist_position",
+                        "position",
+                    ]:
+                        print("  {}: {}".format(item.capitalize(), track_info[item]))
             else:
                 # Assume it's a radio stream
-                print("Playing Stream: {}".format(output["title"]))
+                print(
+                    "Stream is {}:\n  Title = {}\n  URI = {}".format(
+                        state, track_info["title"], track_info["uri"]
+                    )
+                )
+        # Normal track
         else:
-            for item in sorted(output):
+            print("Track is {}:".format(state))
+            for item in sorted(track_info):
                 if item not in ["metadata", "uri", "album_art"]:
-                    print("  {}: {}".format(item.capitalize(), output[item]))
+                    print("  {}: {}".format(item.capitalize(), track_info[item]))
     return True
 
 
@@ -606,6 +624,9 @@ def group_or_pair(speaker, action, args, soco_function, use_local_speaker_list):
         error_and_exit("Pairing operations require SoCo v0.20 or greater")
         return False
     speaker2 = get_speaker(args[0], use_local_speaker_list)
+    if speaker == speaker2:
+        error_and_exit("Speakers are the same")
+        return False
     getattr(speaker, soco_function)(speaker2)
     return True
 
@@ -616,7 +637,6 @@ def operate_on_all(speaker, action, args, soco_function, use_local_speaker_list)
     for zone in zones:
         if zone.is_visible:
             try:
-                # zone.unjoin()
                 getattr(zone, soco_function)()
             except:
                 # Ignore errors here; don't want to halt on
@@ -766,60 +786,48 @@ def save_queue(speaker, action, args, soco_function, use_local_speaker_list):
 @one_parameter
 def seek(speaker, action, args, soco_function, use_local_speaker_list):
     try:
-        time_point = int(convert_to_seconds(args[0]))
-        logging.info("Seek point is {}s".format(time_point))
-        hours = time_point // 3600
-        time_point = time_point - hours * 3600
-        minutes = time_point // 60
-        time_point = time_point - minutes * 60
-        seconds = time_point
-        seek_string = (
-            str(hours).zfill(2)
-            + ":"
-            + str(minutes).zfill(2)
-            + ":"
-            + str(seconds).zfill(2)
-        )
-        logging.info("Seek point is '{}'".format(seek_string))
-        speaker.seek(seek_string)
+        seconds = convert_to_seconds(args[0])
+    except ValueError:
+        parameter_type_error(action, "a valid time format")
+        return False
+    if seconds < 0:
+        parameter_type_error(action, "cannot seek to before start of track")
+        return False
+    seek_point = str(timedelta(seconds=seconds))
+    logging.info("Seek point is {}".format(seek_point))
+    try:
+        # seek() will handle out-of-bounds
+        speaker.seek(seek_point)
     except:
-        parameter_type_error(action, "time value on a seekable source")
+        parameter_type_error(action, "valid time value on a seekable source")
         return False
     return True
 
 
 @one_parameter
 def seek_forward(speaker, action, args, soco_function, use_local_speaker_list):
+    # Calculate the time increment
+    increment = int(convert_to_seconds(args[0]))  # Integer number of seconds
+    if increment < 0:
+        parameter_type_error(action, "a positive time increment")
+        return False
+    logging.info("Seeking forward by {}s".format(increment))
+
+    # Get the current position
+    current_position = speaker.get_current_track_info()["position"]
+    logging.info("Current playback position is '{}'".format(current_position))
+    h, m, s = [int(s) for s in current_position.split(":")]
+
+    td_current = timedelta(hours=h, minutes=m, seconds=s)
+    td_increment = timedelta(seconds=increment)
+    td_new_str = str(td_current + td_increment)
+    logging.info(
+        "Seeking forward to position '{}' ... note: might hit end of track".format(
+            td_new_str
+        )
+    )
     try:
-        # Calculate the time increment
-        increment = int(convert_to_seconds(args[0]))  # Integer number of seconds
-        logging.info("Seeking forward by {}s".format(increment))
-        current_position = speaker.get_current_track_info()["position"]
-        logging.info("Current playback position is '{}'".format(current_position))
-        inc_hours = increment // 3600
-        increment = increment - inc_hours * 3600
-        inc_minutes = increment // 60
-        increment = increment - inc_minutes * 60
-        inc_seconds = increment
-        current_position = current_position.split(":")
-        seconds = (int(current_position[2]) + inc_seconds) % 60
-        inc_minutes = inc_minutes + (int(current_position[2]) + inc_seconds) // 60
-        minutes = (int(current_position[1]) + inc_minutes) % 60
-        inc_hours = inc_hours + (int(current_position[1]) + inc_minutes) // 60
-        hours = int(current_position[0]) + inc_hours
-        seek_string = (
-            str(hours).zfill(2)
-            + ":"
-            + str(minutes).zfill(2)
-            + ":"
-            + str(seconds).zfill(2)
-        )
-        logging.info(
-            "Seeking forward to position '{}' ... might hit end of track".format(
-                seek_string
-            )
-        )
-        speaker.seek(seek_string)
+        speaker.seek(td_new_str)
     except:
         parameter_type_error(action, "time increment on a seekable source")
         return False
@@ -828,35 +836,28 @@ def seek_forward(speaker, action, args, soco_function, use_local_speaker_list):
 
 @one_parameter
 def seek_back(speaker, action, args, soco_function, use_local_speaker_list):
+    # Calculate the time increment
+    increment = int(convert_to_seconds(args[0]))  # Integer number of seconds
+    if increment < 0:
+        parameter_type_error(action, "a positive time increment")
+        return False
+    logging.info("Seeking backward by {}s".format(increment))
+
+    # Get the current position
+    current_position = speaker.get_current_track_info()["position"]
+    logging.info("Current playback position is '{}'".format(current_position))
+    h, m, s = [int(s) for s in current_position.split(":")]
+
+    td_current = timedelta(hours=h, minutes=m, seconds=s)
+    td_increment = timedelta(seconds=increment)
+    if td_current - td_increment < timedelta():
+        logging.info("Cannot seek beyond start of track ... seek to start instead")
+        td_new_str = "00:00:00"
+    else:
+        td_new_str = str(td_current - td_increment)
+    logging.info("Seeking backward to position '{}'".format(td_new_str))
     try:
-        increment = int(convert_to_seconds(args[0]))  # Integer number of seconds
-        current_position = speaker.get_current_track_info()["position"]
-        if increment > int(convert_to_seconds(current_position)):
-            logging.info("Cannot seek beyond start of track ... seek to start instead")
-            speaker.seek("00:00:00")
-            return True
-        logging.info("Current playback position is '{}'".format(current_position))
-        logging.info("Seeking backward by {}s".format(increment))
-        inc_hours = increment // 3600
-        increment = increment - inc_hours * 3600
-        inc_minutes = increment // 60
-        increment = increment - inc_minutes * 60
-        inc_seconds = increment
-        current_position = current_position.split(":")
-        seconds = (int(current_position[2]) - inc_seconds) % 60
-        inc_minutes = inc_minutes - (int(current_position[2]) - inc_seconds) // 60
-        minutes = (int(current_position[1]) - inc_minutes) % 60
-        inc_hours = inc_hours - (int(current_position[1]) - inc_minutes) // 60
-        hours = int(current_position[0]) - inc_hours
-        seek_string = (
-            str(hours).zfill(2)
-            + ":"
-            + str(minutes).zfill(2)
-            + ":"
-            + str(seconds).zfill(2)
-        )
-        logging.info("Seeking backward to position '{}'".format(seek_string))
-        speaker.seek(seek_string)
+        speaker.seek(td_new_str)
     except:
         parameter_type_error(action, "time increment on a seekable source")
         return False
@@ -913,6 +914,7 @@ def list_playlist_tracks(speaker, action, args, soco_function, use_local_speaker
         )
         print_tracks(tracks)
         print()
+        save_search(tracks)
         return True
     else:
         error_and_exit("Playlist '{}' not found".format(args[0]))
@@ -1201,6 +1203,8 @@ def wait_stopped_for(speaker, action, args, soco_function, use_local_speaker_lis
         sub = speaker.avTransport.subscribe(auto_renew=True)
     except Exception as e:
         error_and_exit("Exception {}".format(e))
+
+    playing_states = ["PLAYING", "TRANSITIONING"]
     while True:
         try:
             # ToDo: Remove temporary fix for CTRL-C not exiting
@@ -1211,8 +1215,8 @@ def wait_stopped_for(speaker, action, args, soco_function, use_local_speaker_lis
                     event.variables["transport_state"]
                 )
             )
-            if event.variables["transport_state"] not in ["PLAYING", "TRANSITIONING"]:
-                logging.info("Speaker is not 'PLAYING' or 'TRANSITIONING'")
+            if event.variables["transport_state"] not in playing_states:
+                logging.info("Speaker is not in states {}".format(playing_states))
                 sub.unsubscribe()
                 # ToDo: Should really return here and do this some other way ...
                 #       this is what's requiring the SIGKILL
@@ -1231,19 +1235,24 @@ def wait_stopped_for(speaker, action, args, soco_function, use_local_speaker_lis
                         "current_transport_state"
                     ]
                     logging.info("Transport state = '{}'".format(state))
-                    if state == "PLAYING":
+                    if state in playing_states:
                         # Restart the timer
                         start_time = current_time
+                    remaining_time = duration - (current_time - start_time)
                     logging.info(
-                        "Elapsed since not 'PLAYING' = {}s, Total elapsed = {}s".format(
+                        "Elapsed since last 'STOPPED' = {}s | total elapsed = {}s | remaining = {}s".format(
                             int(current_time - start_time),
                             int(current_time - original_start_time),
+                            int(remaining_time),
                         )
                     )
-                    time.sleep(poll_interval)
+                    if remaining_time <= poll_interval:
+                        time.sleep(remaining_time)
+                    else:
+                        time.sleep(poll_interval)
                     current_time = time.time()
                 logging.info(
-                    "Timer expired after not 'PLAYING' for {}s, total elapsed = {}s".format(
+                    "Timer expired after 'STOPPED' for {}s | total elapsed = {}s".format(
                         int(current_time - start_time),
                         int(current_time - original_start_time),
                     )
@@ -1506,6 +1515,9 @@ def transfer_playback(speaker, action, args, soco_function, use_local_speaker_li
         error_and_exit("Speaker '{}' is not a coordinator".format(speaker.player_name))
         return False
     speaker2 = get_speaker(args[0], use_local_speaker_list)
+    if speaker == speaker2:
+        error_and_exit("Source and target speakers are the same")
+        return False
     if speaker2:
         speaker2.join(speaker)
         speaker.unjoin()
@@ -1537,7 +1549,7 @@ def last_search(speaker, action, args, soco_function, use_local_speaker_list):
                 print_albums(items)
             # 'artists' search_type is used for tracks when 'tracks_in_album' has
             #  been used for the search
-            elif items.search_type in ["tracks", "artists"]:
+            elif items.search_type in ["tracks", "artists", "browse"]:
                 print_tracks(items)
             print()
     else:
@@ -1581,9 +1593,11 @@ def queue_search_result_number(
             position = (
                 int(speaker.get_current_track_info()["playlist_position"]) + offset
             )
+        elif args[1].lower() in ["first", "start"]:
+            position = 1
         else:
             error_and_exit(
-                "If supplied, second parameter for '{}' must be 'next/play_next'".format(
+                "If supplied, second parameter for '{}' must be 'next/play_next' or 'first/start'".format(
                     action
                 )
             )
@@ -1604,6 +1618,41 @@ def cue_favourite_radio_station(
     return cue_favourite(speaker, action, args, soco_function, use_local_speaker_list)
 
 
+def battery(speaker, action, args, soco_function, use_local_speaker_list):
+    info = speaker.get_speaker_info()
+    logging.info("Retrieved speaker info: {}".format(info))
+    if info["model_name"] not in ["Sonos Move"]:
+        error_and_exit(
+            "Speaker '{}' doesn't have a battery".format(speaker.player_name)
+        )
+        return False
+
+    # Retrieve information from the speaker's support URL
+    url = "http://" + speaker.ip_address + ":1400/support/review"
+    try:
+        response = requests.get(url)
+    except:
+        error_and_exit("Failed to retrieve speaker information")
+        return False
+    if response.status_code != 200:
+        error_and_exit("Failed to retrieve speaker information")
+        return False
+
+    # Traverse XML to obtain the battery information
+    try:
+        data = xmltodict.parse(response.text)
+        zp_list = data["ZPNetworkInfo"]["ZPSupportInfo"]
+        for zp in zp_list:
+            if zp["ZPInfo"]["IPAddress"] == speaker.ip_address:
+                battery_status = zp["LocalBatteryStatus"]["Data"]
+                for item in battery_status:
+                    print("  {}: {}".format(item["@name"], item["#text"]))
+                return True
+    except:
+        error_and_exit("Error in the information returned by the speaker")
+        return False
+
+
 def process_action(speaker, action, args, use_local_speaker_list):
     sonos_function = actions.get(action, None)
     if sonos_function:
@@ -1835,4 +1884,5 @@ actions = {
     "cue_favourite_radio_station": SonosFunction(cue_favourite_radio_station, ""),
     "cue_favorite_radio_station": SonosFunction(cue_favourite_radio_station, ""),
     "cfrs": SonosFunction(cue_favourite_radio_station, ""),
+    "battery": SonosFunction(battery, ""),
 }
