@@ -5,6 +5,9 @@ import pprint
 import time
 from signal import SIGINT, signal
 
+# Temporary Python 3.9 patch until SoCo 0.21 arrives
+from threading import Thread
+
 from .action_processor import process_action
 from .speakers import Speakers
 from .utils import (
@@ -22,6 +25,8 @@ from .utils import (
     version,
 )
 
+Thread.isAlive = Thread.is_alive
+
 # Globals
 pp = pprint.PrettyPrinter(width=100)
 
@@ -33,7 +38,7 @@ def main():
     # Create the argument parser
     parser = argparse.ArgumentParser(
         prog="sonos",
-        usage="%(prog)s SPEAKER_NAME_OR_IP ACTION <parameters> <: ...>",
+        usage="%(prog)s SPEAKER_NAME_OR_IP ACTION <parameters> < : ...>",
         description="Command line utility for controlling Sonos speakers",
     )
     # A variable number of arguments depending on the action
