@@ -127,7 +127,8 @@ class Speakers:
         if self._speakers:
             if not os.path.exists(self._save_directory):
                 os.mkdir(self._save_directory)
-            pickle.dump(self._speakers, open(self.save_pathname, "wb"))
+            with open(self.save_pathname, "wb") as f:
+                pickle.dump(self._speakers, f)
             return True
         else:
             return False
@@ -136,7 +137,8 @@ class Speakers:
         """Loads a saved speaker list"""
         if os.path.exists(self.save_pathname):
             try:
-                self._speakers = pickle.load(open(self.save_pathname, "rb"))
+                with open(self.save_pathname, "rb") as f:
+                    self._speakers = pickle.load(f)
             except:
                 return False
             return True
@@ -171,15 +173,16 @@ class Speakers:
                     network_ip = ipaddress.ip_network(ip.ip)
                     if network_ip.is_private and not network_ip.is_loopback:
                         # Constrain the size of network that will be searched
-                        if ip.network_prefix < self._min_netmask:
+                        netmask = ip.network_prefix
+                        if netmask < self._min_netmask:
                             logging.info(
-                                "Constraining netmask={} to {}".format(
-                                    ip.network_prefix, self._min_netmask
+                                "{}: Constraining netmask={} to {}".format(
+                                    ip.ip, ip.network_prefix, self._min_netmask
                                 )
                             )
-                            ip.network_prefix = self._min_netmask
+                            netmask = self._min_netmask
                         network = ipaddress.ip_network(
-                            ip.ip + "/" + str(ip.network_prefix), False
+                            ip.ip + "/" + str(netmask), False
                         )
                         ipv4_net_list.add(network)
         self._networks = list(ipv4_net_list)
@@ -197,17 +200,17 @@ class Speakers:
     @staticmethod
     def check_ip_and_port(ip, port, timeout):
         """Determine if a port is open"""
-        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        s.settimeout(timeout)
-        if s.connect_ex((ip, port)) == 0:
-            return True
-        else:
-            logging.debug(
-                "Socket connection to {}:{} timed out after {}s".format(
-                    ip, port, timeout
+        with (socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as socket_:
+            socket_.settimeout(timeout)
+            if socket_.connect_ex((ip, port)) == 0:
+                return True
+            else:
+                logging.debug(
+                    "Socket connection to {}:{} timed out after {}s".format(
+                        ip, port, timeout
+                    )
                 )
-            )
-            return False
+                return False
 
     @staticmethod
     def get_sonos_device_data(ip_addr):
@@ -238,7 +241,15 @@ class Speakers:
                 ip_addr = ip_set.pop()
             except KeyError:
                 break
-            if Speakers.check_ip_and_port(str(ip_addr), 1400, socket_timeout):
+            try:
+                check = Speakers.check_ip_and_port(str(ip_addr), 1400, socket_timeout)
+            except OSError:
+                # Return the ip address to the set, and break out of this thread
+                # This has the effect of reducing the number of threads running
+                logging.info("OSError exception from socket calls")
+                ip_set.add(ip_addr)
+                break
+            if check:
                 device = Speakers.get_sonos_device_data(ip_addr)
                 if device:
                     sonos_devices.append(device)
